@@ -15,7 +15,13 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 
-import collections
+# TODO(smcginnis) update this once six has support for collections.abc
+# (https://github.com/benjaminp/six/pull/241) or clean up once we drop py2.7.
+try:
+    from collections.abc import Mapping
+except ImportError:
+    from collections import Mapping
+
 import copy
 import logging
 import sys
@@ -26,8 +32,6 @@ from oslo_utils import timeutils
 import six
 
 import oslo_messaging
-from oslo_messaging._i18n import _
-from oslo_messaging._i18n import _LE, _LW
 from oslo_messaging import _utils as utils
 
 LOG = logging.getLogger(__name__)
@@ -75,7 +79,7 @@ _REMOTE_POSTFIX = '_Remote'
 
 
 class RPCException(Exception):
-    msg_fmt = _("An unknown RPC related exception occurred.")
+    msg_fmt = "An unknown RPC related exception occurred."
 
     def __init__(self, message=None, **kwargs):
         self.kwargs = kwargs
@@ -87,8 +91,8 @@ class RPCException(Exception):
             except Exception:
                 # kwargs doesn't match a variable in the message
                 # log the issue and the kwargs
-                LOG.exception(_LE('Exception in string format operation, '
-                                  'kwargs are:'))
+                LOG.exception('Exception in string format operation, '
+                              'kwargs are:')
                 for name, value in kwargs.items():
                     LOG.error("%s: %s", name, value)
                 # at least get the core message out if something happened
@@ -103,9 +107,9 @@ class Timeout(RPCException):
     This exception is raised if the rpc_response_timeout is reached while
     waiting for a response from the remote side.
     """
-    msg_fmt = _('Timeout while waiting on RPC response - '
-                'topic: "%(topic)s", RPC method: "%(method)s" '
-                'info: "%(info)s"')
+    msg_fmt = ('Timeout while waiting on RPC response - '
+               'topic: "%(topic)s", RPC method: "%(method)s" '
+               'info: "%(info)s"')
 
     def __init__(self, info=None, topic=None, method=None):
         """Initiates Timeout object.
@@ -120,31 +124,31 @@ class Timeout(RPCException):
         self.method = method
         super(Timeout, self).__init__(
             None,
-            info=info or _('<unknown>'),
-            topic=topic or _('<unknown>'),
-            method=method or _('<unknown>'))
+            info=info or '<unknown>',
+            topic=topic or '<unknown>',
+            method=method or '<unknown>')
 
 
 class DuplicateMessageError(RPCException):
-    msg_fmt = _("Found duplicate message(%(msg_id)s). Skipping it.")
+    msg_fmt = "Found duplicate message(%(msg_id)s). Skipping it."
 
 
 class InvalidRPCConnectionReuse(RPCException):
-    msg_fmt = _("Invalid reuse of an RPC connection.")
+    msg_fmt = "Invalid reuse of an RPC connection."
 
 
 class UnsupportedRpcVersion(RPCException):
-    msg_fmt = _("Specified RPC version, %(version)s, not supported by "
-                "this endpoint.")
+    msg_fmt = ("Specified RPC version, %(version)s, not supported by "
+               "this endpoint.")
 
 
 class UnsupportedRpcEnvelopeVersion(RPCException):
-    msg_fmt = _("Specified RPC envelope version, %(version)s, "
-                "not supported by this endpoint.")
+    msg_fmt = ("Specified RPC envelope version, %(version)s, "
+               "not supported by this endpoint.")
 
 
 class RpcVersionCapError(RPCException):
-    msg_fmt = _("Specified RPC version cap, %(version_cap)s, is too low")
+    msg_fmt = "Specified RPC version cap, %(version_cap)s, is too low"
 
 
 class Connection(object):
@@ -230,7 +234,7 @@ def deserialize_remote_exception(data, allowed_remote_exmods):
 
         failure = klass(*failure.get('args', []), **failure.get('kwargs', {}))
     except (AttributeError, TypeError, ImportError) as error:
-        LOG.warning(_LW("Failed to rebuild remote exception due to error: %s"),
+        LOG.warning("Failed to rebuild remote exception due to error: %s",
                     six.text_type(error))
         return oslo_messaging.RemoteError(name, failure.get('message'), trace)
 
@@ -399,11 +403,32 @@ class ConnectionContext(Connection):
         """Create a new connection, or get one from the pool."""
         self.connection = None
         self.connection_pool = connection_pool
+
+        # NOTE(sileht): Even if rabbit only has one Connection class this
+        # connection can be used for only one of two purposes:
+        #
+        # * receiving messages from the broker (read actions on the socket)
+        # * sending messages to the broker (write actions on the socket)
+        #
+        # Using one Connection class instance for both purposes will result in
+        # eventlet complaining about multiple greenthreads that read/write the
+        # same fd concurrently. This is because 'send' and 'listen' run in
+        # different greenthreads and the code inside a connection class is not
+        # concurrency safe. The 'purpose' parameter ensures that no connection
+        # is used for both sending and receiving messages.
+        #
+        # The rabbitmq driver allocates connections in the following manner:
+        # * driver.listen*(): each call creates a new dedicated
+        #   'PURPOSE_LISTEN' connection for the listener
+        # * driver.send*(): senders are assigned a connection from a pool of
+        #   'PURPOSE_SEND' connections maintained by the driver.
+        # * One 'PURPOSE_LISTEN' connection is dedicated to waiting for replies
+        #   to rpc calls.
+
         pooled = purpose == PURPOSE_SEND
         if pooled:
             self.connection = connection_pool.get()
         else:
-            # a non-pooled connection is requested, so create a new connection
             self.connection = connection_pool.create(purpose)
         self.pooled = pooled
         self.connection.pooled = pooled
@@ -423,7 +448,7 @@ class ConnectionContext(Connection):
                 try:
                     self.connection.reset()
                 except Exception:
-                    LOG.exception(_LE("Fail to reset the connection, drop it"))
+                    LOG.exception("Fail to reset the connection, drop it")
                     try:
                         self.connection.close()
                     except Exception as exc:
@@ -460,7 +485,7 @@ class ConnectionContext(Connection):
             raise InvalidRPCConnectionReuse()
 
 
-class ConfigOptsProxy(collections.Mapping):
+class ConfigOptsProxy(Mapping):
     """Proxy for oslo_config.cfg.ConfigOpts.
 
     Values from the query part of the transport url (if they are both present
@@ -497,7 +522,7 @@ class ConfigOptsProxy(collections.Mapping):
     def __len__(self):
         return len(self._conf)
 
-    class GroupAttrProxy(collections.Mapping):
+    class GroupAttrProxy(Mapping):
         """Internal helper proxy for oslo_config.cfg.ConfigOpts.GroupAttr."""
 
         _VOID_MARKER = object()
