@@ -26,7 +26,6 @@ import six
 from stevedore import extension
 from stevedore import named
 
-from oslo_messaging._i18n import _LE
 from oslo_messaging import serializer as msg_serializer
 from oslo_messaging import transport as msg_transport
 
@@ -140,8 +139,7 @@ class Driver(object):
         pass
 
 
-def get_notification_transport(conf, url=None,
-                               allowed_remote_exmods=None, aliases=None):
+def get_notification_transport(conf, url=None, allowed_remote_exmods=None):
     """A factory method for Transport objects for notifications.
 
     This method should be used for notifications, in case notifications are
@@ -164,15 +162,13 @@ def get_notification_transport(conf, url=None,
                                   transport will deserialize remote exceptions
                                   from
     :type allowed_remote_exmods: list
-    :param aliases: A map of transport alias to transport name
-    :type aliases: dict
     """
     conf.register_opts(_notifier_opts,
                        group='oslo_messaging_notifications')
     if url is None:
         url = conf.oslo_messaging_notifications.transport_url
     return msg_transport._get_transport(
-        conf, url, allowed_remote_exmods, aliases,
+        conf, url, allowed_remote_exmods,
         transport_cls=msg_transport.NotificationTransport)
 
 
@@ -314,9 +310,9 @@ class Notifier(object):
             try:
                 ext.obj.notify(ctxt, msg, priority, retry or self.retry)
             except Exception as e:
-                _LOG.exception(_LE("Problem '%(e)s' attempting to send to "
-                                   "notification system. Payload=%(payload)s"),
-                               dict(e=e, payload=payload))
+                _LOG.exception("Problem '%(e)s' attempting to send to "
+                               "notification system. Payload=%(payload)s",
+                               {'e': e, 'payload': payload})
 
         if self._driver_mgr.extensions:
             self._driver_mgr.map(do_notify)
