@@ -21,7 +21,6 @@ from six import moves
 
 import oslo_messaging
 from oslo_messaging._drivers.kafka_driver import kafka_options
-from oslo_messaging._drivers.zmq_driver import zmq_options
 from oslo_messaging.notify import notifier
 from oslo_messaging.tests import utils as test_utils
 
@@ -304,8 +303,6 @@ class SkipIfNoTransportURL(test_utils.BaseTestCase):
         driver = os.environ.get("TRANSPORT_DRIVER")
         if driver:
             self.url = os.environ.get('PIFPAF_URL')
-            if driver == "pika" and self.url:
-                self.url = self.url.replace("rabbit://", "pika://")
         else:
             self.url = os.environ.get('TRANSPORT_URL')
 
@@ -314,39 +311,8 @@ class SkipIfNoTransportURL(test_utils.BaseTestCase):
 
         transport_url = oslo_messaging.TransportURL.parse(conf, self.url)
 
-        zmq_options.register_opts(conf, transport_url)
-
-        zmq_matchmaker = os.environ.get('ZMQ_MATCHMAKER')
-        if zmq_matchmaker:
-            self.config(rpc_zmq_matchmaker=zmq_matchmaker,
-                        group="oslo_messaging_zmq")
-        zmq_ipc_dir = os.environ.get('ZMQ_IPC_DIR')
-        if zmq_ipc_dir:
-            self.config(group="oslo_messaging_zmq",
-                        rpc_zmq_ipc_dir=zmq_ipc_dir)
-        zmq_redis_port = os.environ.get('ZMQ_REDIS_PORT')
-        if zmq_redis_port:
-            self.config(port=zmq_redis_port,
-                        check_timeout=10000,
-                        wait_timeout=1000,
-                        group="matchmaker_redis")
-        zmq_use_pub_sub = os.environ.get('ZMQ_USE_PUB_SUB')
-        zmq_use_router_proxy = os.environ.get('ZMQ_USE_ROUTER_PROXY')
-        zmq_use_acks = os.environ.get('ZMQ_USE_ACKS')
-        self.config(use_pub_sub=zmq_use_pub_sub,
-                    use_router_proxy=zmq_use_router_proxy,
-                    rpc_use_acks=zmq_use_acks,
-                    group='oslo_messaging_zmq')
-        zmq_use_dynamic_connections = \
-            os.environ.get('ZMQ_USE_DYNAMIC_CONNECTIONS')
-        self.config(use_dynamic_connections=zmq_use_dynamic_connections,
-                    group='oslo_messaging_zmq')
-
         kafka_options.register_opts(conf, transport_url)
 
-        self.config(producer_batch_size=0,
-                    group='oslo_messaging_kafka')
-
 
 class NotificationFixture(fixtures.Fixture):
     def __init__(self, conf, url, topics, batch=None):
